//========================================================================================
//
//	File:				Content.h
//	Release Version:	$ ODF 1 $
//
//	Author:				Mary Boetcher
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef BMPCONT_H
#define BMPCONT_H

// ----- FrameWork Includes -----

#ifndef FWCONTNT_H
#include "FWContnt.h"
#endif

#ifndef FWGRREF_H
#include "FWGrRef.h"
#endif

#ifndef FWBITMAP_H
#include "FWBitmap.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class CBitmapPart;
class CBitmapSelection;

//========================================================================================
//	class CBitmapContent
//========================================================================================

class CBitmapContent : public FW_CContent
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_DECLARE_AUTO(CBitmapContent)
	
	CBitmapContent(Environment* ev, CBitmapPart* part);

	virtual ~CBitmapContent();

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		Externalize(Environment* ev,
									 ODStorageUnit* destinationSU,
									 FW_EStorageKinds storageKind,
									 FW_CCloneInfo* cloneInfo);
	virtual FW_Boolean	Internalize(Environment* ev,
									 ODStorageUnit* storageUnit, 
									 FW_EStorageKinds storageKind,
									 FW_CCloneInfo* cloneInfo);
											 
//----------------------------------------------------------------------------------------
//	Private methods
//
public:
	void				ExternalizeBitmapShape(Environment* ev, ODStorageUnit* storageUnit, const FW_CBitmap& bitmap);
	void				InternalizeBitmapShape(Environment* ev, ODStorageUnit* storageUnit);

#ifdef FW_BUILD_MAC
	void				ExternalizePICTData(Environment* ev, ODStorageUnit* storageUnit, const FW_CBitmap& bitmap);

	void				InternalizePICTData(Environment* ev, ODStorageUnit* storageUnit);
	void				InternalizePICTFile(Environment* ev, ODStorageUnit* storageUnit);
	void				MacSetBitmap(Environment* ev, FW_PlatformPict picHandle);
#endif
	
//----------------------------------------------------------------------------------------
//	Getters & Setters
//
public:
	FW_CBitmap 			GetBitmap(Environment* ev) const;
	void 				SetBitmap(Environment* ev, FW_CBitmap bitmap);

	void				UseDefaultBitmap(Environment* ev);

//----------------------------------------------------------------------------------------
//	Data Members
//
protected:
	FW_CBitmap			fBitmap;		// counted pointer to the bitmap
	CBitmapPart*		fBitmapPart;
};

//========================================================================================
//	class CBitmapSelectionContent
//========================================================================================

class CBitmapSelectionContent : public FW_CContent
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	CBitmapSelectionContent(Environment* ev, 
							CBitmapPart* part, 
							CBitmapSelection* selection, 
							CBitmapContent* bitmapContent);

	virtual ~CBitmapSelectionContent();

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		Externalize(Environment* ev,
									 ODStorageUnit* storageUnit,
									 FW_EStorageKinds storageKind,
									 FW_CCloneInfo* cloneInfo);
	virtual FW_Boolean	Internalize(Environment* ev,
									 ODStorageUnit* storageUnit, 
									 FW_EStorageKinds storageKind,
									 FW_CCloneInfo* cloneInfo);

	virtual ODShape*	CreateDataFrameShape(Environment* ev) const;

//----------------------------------------------------------------------------------------
//	New API
//
public:
	void				GetSelectRect(FW_CPlatformRect& rect) const
							{rect = fSelectRect;}
	void				SetSelectRect(const FW_CPlatformRect& rect)
							{fSelectRect = rect;}
							

	void				SelectEntireBitmap(Environment* ev);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	CBitmapPart*		fBitmapPart;
	CBitmapContent*		fBitmapContent;
	CBitmapSelection*	fBitmapSelection;
	FW_CPlatformRect	fSelectRect;
};


#endif