//========================================================================================
//
//	File:				Frame.h
//	Release Version:	$ ODF 1 $
//
//	Author:				Henri Lamiraux
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef BMPFRAME_H
#define BMPFRAME_H

// ----- Framework Includes -----

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWFDRGDRP_H
#include "FWDrgDrp.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CGrowBox;
class CBitmapPart;
class CBitmapSelection;
class CBitmapContent;

//========================================================================================
//	class CBitmapFrame
//========================================================================================

class CBitmapFrame : public FW_CFrame, public FW_MDraggableFrame, public FW_MDroppableFrame
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_DECLARE_AUTO(CBitmapFrame)
	
	CBitmapFrame(Environment* ev, 
				ODFrame* odFrame, 
				FW_CPresentation* presentation, 
				CBitmapPart* bitmapPart,
				CBitmapContent* bitmapContent);
	virtual ~ CBitmapFrame();
	
//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual FW_Boolean				DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);
	virtual FW_Boolean				DoAdjustMenus(Environment* ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);

	virtual void					FocusStateChanged(Environment* ev, ODTypeToken focus, FW_Boolean newState, ODFrame* newOwner);
	virtual void 					FrameShapeChanged(Environment* ev);
	
	virtual ODShape*				AdjustUsedShape(Environment* ev, ODShape* suggestedUsedShape);
	virtual ODShape*				AdjustActiveShape(Environment* ev, ODFacet* facet, ODShape* suggestedActiveShape);

	virtual void					AdjustZoomedWindowSize(Environment *ev, FW_CPoint& proposedSize);

	virtual void 					Draw(Environment* ev, ODFacet* odFacet, ODShape* invalidShape);
	virtual void					BuildThumbnail(Environment* ev, FW_CGraphicContext& gc, FW_CRect& usedRect);
			
	virtual FW_Boolean 				DoMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	
#ifdef FW_BUILD_MAC
	virtual ODDragResult			CanAcceptDrop(Environment* ev, ODDragItemIterator* dragInfo);
#endif

	virtual FW_Boolean				DoIdle(Environment* ev, const FW_CNullEvent& theNullEvent);
	
	virtual void					CreateSubViews(Environment* ev);

	// ----- Commands -----
	virtual FW_CClipboardCommand*	NewClipboardCommand(Environment* ev, ODCommandID commandID);

	virtual FW_CDropCommand* 		NewDropCommand(Environment *ev, 
										FW_CFrame* frame,
										ODDragItemIterator* dropInfo, 
										ODFacet* facet, 
										const FW_CPoint& dropPoint);

	virtual FW_CDragCommand* 		NewDragCommand(Environment *ev, 
										FW_CFrame* theFrame,
										const FW_CMouseEvent& theMouseEvent);

	// ----- External/Internalize -----
	virtual void					ExternalizeFrame(Environment* ev, ODStorageUnitView* storageUnitView);
	virtual void					InternalizeFrame(Environment* ev, ODStorageUnitView* storageUnitView);

//----------------------------------------------------------------------------------------
//	New API
//
public:
	FW_CPoint						GetZoomRatio(Environment* ev) const;
	void							AdjustFrameSize(Environment* ev);

	void							CalcUsedRect(Environment* ev, FW_CRect& newUsedRect) const;
	
	ODCommandID						GetChoosenSize() const
										{return fChoosenSize;}
								
//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	CBitmapContent* 		fBitmapContent;		// Pointer to my content
	ODCommandID				fChoosenSize;		// Choosen Size (50%, 100%, 200%, or fit to frame)
	FW_CRect				fFrameRect;			// The lastest frame shape rect
	CBitmapSelection*		fSelection;
	FW_CIdler*				fIdler;
};

#endif
